@@ -25,6 +25,7 @@
 #include "totem.h"
 #include "totem-private.h"
 #include "totem-session.h"
+#include "totem-uri.h"
 
 #ifndef HAVE_GTK_ONLY
 
@@ -36,12 +37,11 @@ totem_session_create_key (void)
 {
 	char *filename, *path;
 
-	filename = g_strdup_printf ("totem-%d-%d-%u.pls",
+	filename = g_strdup_printf ("playlist-%d-%d-%u.pls",
 			(int) getpid (),
 			(int) time (NULL),
 			g_random_int ());
-	path = g_build_filename (g_get_home_dir (),
-			".gnome2", filename, NULL);
+	path = g_build_filename (totem_dot_dir (), filename, NULL);
 	g_free (filename);
 
 	return path;
@@ -54,7 +54,10 @@ totem_save_yourself_cb (GnomeClient *client, int phase, GnomeSaveStyle style,
 {
 	char *argv[] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL };
 	int i = 0;
-	char *path_id, *current, *seek;
+	char *path_id, *current, *seek, *uri;
+
+	if (style == GNOME_SAVE_GLOBAL)
+		return TRUE;
 
 	path_id = totem_session_create_key ();
 	totem_playlist_save_current_playlist (totem->playlist, path_id);
@@ -76,7 +79,9 @@ totem_save_yourself_cb (GnomeClient *client, int phase, GnomeSaveStyle style,
 	argv[i++] = current;
 	argv[i++] = "--seek";
 	argv[i++] = seek;
-	argv[i++] = path_id;
+
+	uri = g_filename_to_uri (path_id, NULL, NULL);
+	argv[i++] = uri;
 
 	gnome_client_set_clone_command (client, i, argv);
 	gnome_client_set_restart_command (client, i, argv);
@@ -84,6 +89,7 @@ totem_save_yourself_cb (GnomeClient *client, int phase, GnomeSaveStyle style,
 	g_free (path_id);
 	g_free (current);
 	g_free (seek);
+	g_free (uri);
 
 	return TRUE;
 }
@@ -98,6 +104,7 @@ void
 totem_session_setup (Totem *totem, char **argv)
 {
 	GnomeClient *client;
+	GnomeClientFlags flags;
 
 	totem->argv0 = argv[0];
 
@@ -106,28 +113,27 @@ totem_session_setup (Totem *totem, char **argv)
 			G_CALLBACK (totem_save_yourself_cb), totem);
 	g_signal_connect (G_OBJECT (client), "die",
 			G_CALLBACK (totem_client_die_cb), totem);
+
+	flags = gnome_client_get_flags (client);
+	if (flags & GNOME_CLIENT_RESTORED)
+		totem->session_restored = TRUE;
 }
 
 void
-totem_session_restore (Totem *totem, char **argv)
+totem_session_restore (Totem *totem, char **filenames)
 {
-	char *path, *mrl;
+	char *mrl, *uri;
 
-	g_return_if_fail (argv[0] != NULL);
-	path = argv[0];
-
-	if (g_file_test (path, G_FILE_TEST_EXISTS) == FALSE)
-	{
-		totem_action_set_mrl (totem, NULL);
-		return;
-	}
+	g_return_if_fail (filenames[0] != NULL);
+	uri = filenames[0];
 
 	totem_signal_block_by_data (totem->playlist, totem);
 
-	if (totem_playlist_add_mrl (totem->playlist, path, NULL) == FALSE)
+	if (totem_playlist_add_mrl (totem->playlist, uri, NULL) == FALSE)
 	{
 		totem_signal_unblock_by_data (totem->playlist, totem);
 		totem_action_set_mrl (totem, NULL);
+		g_free (uri);
 		return;
 	}
 
@@ -158,7 +164,8 @@ totem_session_setup (Totem *totem, char **argv)
 {
 }
 
-void totem_action_restore_pl (Totem *totem)
+void
+totem_session_restore (Totem *totem, char **argv)
 {
 }
 
